# -*- coding: utf-8 -*-
"""fluid_design_system implementation of base components

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.registry import yes

from cubicweb.web.views.basecomponents import HeaderComponent
from cubicweb.web.views.basetemplates import TheMainTemplate

from cubicweb.web import component

from cubicweb_fluid_design_system import monkeypatch_default_value

monkeypatch_default_value(component.CtxComponent.render_items, 'klass', u'list-unstyled')


class HideAsidesBar(HeaderComponent):
    """ Hide the left bar """
    __regid__ = 'hide-left-bar'
    __select__ = yes()
    context = 'header-right'
    order = 3
    visible = False
    icon_css_cls = 'glyphicon glyphicon-align-justify'

    def render(self, w):
        define_var = self._cw.html_headers.define_var
        define_var('twbs_col_cls', TheMainTemplate.twbs_col_cls)
        define_var('twbs_col_size', TheMainTemplate.twbs_col_size)
        define_var('twbs_grid_columns', TheMainTemplate.twbs_grid_columns)
        w(u'''<button class="btn btn-default navbar-btn" id="cw-aside-toggle"
              onclick="cw.cubes.squareui.toggleLeftColumn()" title="%(label)s">
              <span class="%(icon_class)s"></span>
              </button>''' % {'icon_class': self.icon_css_cls,
                              'label': self._cw._('collapse boxes')})



import unittest

from cubicweb.devtools.testlib import CubicWebTC
from cubicweb.devtools.htmlparser import XMLValidator


class BoostrapLogFormTemplateTC(CubicWebTC):

    def _login_labels(self):
        valid = self.content_type_validators.get('text/html', XMLValidator)()
        with self.new_access(u'anon').web_request() as req:
            page = valid.parse_string(self.vreg['views'].main_template(req, 'login'))
        return page.find_tag('label')

    def test_label(self):
        self.set_option('allow-email-login', 'yes')
        self.assertEqual(self._login_labels(), ['login or email', 'password'])
        self.set_option('allow-email-login', 'no')
        self.assertEqual(self._login_labels(), ['login', 'password'])


if __name__ == '__main__':
    unittest.main()



"""
:copyright 2012 CreaLibre (Monterrey, MEXICO), all rights reserved.
:contact http://www.crealibre.com/ -- mailto:info@crealibre.com

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
# flake8: noqa

import pathlib


STYLESHEETS = [
    data('css/bootstrap.min.css'),
    data('cubes.bootstrap.css'),
    data('cubicweb.pictograms.css'),
    'https://cdn.jsdelivr.net/npm/@engie-group/fluid-design-system@latest/lib/fluid-design-system.css',
    'https://fonts.googleapis.com/css?family=Material+Icons|Lato:300,400,700,900&display=swap'
]

CW_COMPAT_STYLESHEETS = [data('cubes.bootstrap.cw_compat.css')]

JAVASCRIPTS.extend((data('js/bootstrap.min.js'),
                    data('cubes.bootstrap.js'),
                    data('cubes.squareui.js'),
                    'https://cdn.jsdelivr.net/npm/@engie-group/fluid-design-system@latest/lib/fluid-design-system.js',
                    'https://cdn.jsdelivr.net/npm/@engie-group/fluid-design-system@latest/lib/auto-init.js'))



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of boxes

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb.utils import wrap_on_write
from cubicweb.web.views import boxes

from cubicweb_fluid_design_system import CW_325

boxes.SearchBox.formdef = (
    u'<form action="%(action)s" id="search_box" class="navbar-form" role="search">\n'
    u'  <input id="norql" type="text" accesskey="q" tabindex="%(tabindex1)s"'
    u'    title="search text" value="%(value)s" name="rql"'
    u'    class="search-query form-control" placeholder="%(searchlabel)s"/>\n'
    u'  <input type="hidden" name="__fromsearchbox" value="1" />\n'
    u'  <input type="hidden" name="subvid" value="tsearch" />\n'
    u'</form>\n')


@monkeypatch(boxes.SearchBox)
def render_body(self, w):
    # Don't display search box title, just display the search box body
    if self._cw.form.pop('__fromsearchbox', None):
        rql = self._cw.form.get('rql', '')
    else:
        rql = ''
    w(self.formdef % {'action': self._cw.build_url('view'),
                      'tabindex1': self._cw.next_tabindex() if not CW_325 else u'',
                      'value': xml_escape(rql),
                      'searchlabel': self._cw._('Search')})


@monkeypatch(boxes.ContextualBoxLayout)
def render(self, w):
    if self.init_rendering():
        view = self.cw_extra_kwargs['view']
        w(u'<div class="panel panel-default %s %s" id="%s">' % (self.cssclass, view.cssclass,
                                                                view.domid))
        with wrap_on_write(w,
                           u'<div class="panel-heading"><div class="panel-title">',
                           u'</div></div>') as wow:
            view.render_title(wow)
        w(u'<div class="panel-body">')
        view.render_body(w)
        # We dissapear the boxFooter CSS place holder, as shadows
        # or effect will be made with CSS
        w(u'</div></div>\n')



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of base components

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb.web.views import basecomponents
from cubicweb.uilib import js

from cubicweb_fluid_design_system.views import utils
from cubicweb_fluid_design_system import CW_325


basecomponents.CookieLoginComponent._html = (
    u'<a class="icon-login" data-toggle="modal" href="#loginModal">%s</a>')


@monkeypatch(basecomponents.CookieLoginComponent)
def call(self):
    self.w(self._html % self._cw._('i18n_login_popup'))
    self._cw.add_onload(js.jQuery("body").append(
        self._cw.view('logform', rset=self.cw_rset, id=self.loginboxid,
                      klass='%s' % self.loginboxid, title=True,
                      showmessage=False, showonload=False)))
    self._cw.add_onload('''
       $('#loginModal').on('shown.bs.modal', function () {
           $('#__login:visible').focus();
       });''')


# NOTE: CW 3.18 may introduce render_messages(). This would be the
#       the only method to override
@monkeypatch(basecomponents.ApplicationMessage)  # noqa: F811
def call(self, msg=None):
    if msg is None:
        msg = self._cw.message  # XXX don't call self._cw.message twice
    if msg:
        self.w(u'<div class="alert alert-info" id="%s">'
               u'<button class="close" data-dismiss="alert" type="button">x</button>'
               u' %s</div>' % (self.domid, msg))


@monkeypatch(basecomponents.ApplLogo)
def render(self, w):
    w(u'<a id="logo" href="%s"></a>' % self._cw.base_url())


@monkeypatch(basecomponents.ApplicationName)  # noqa: F811
def render(self, w, **kwargs):
    title = self._cw.property_value('ui.site-title')
    if title:
        w(u'<a class="cw-site-title" href="%s">%s</a>'
          % (self._cw.base_url(), xml_escape(title)))


class BSAuthenticatedUserStatus(basecomponents.AuthenticatedUserStatus):
    divider_html = u'<li class="divider"></li>'

    def render(self, w):
        # display useractions and siteactions
        actions = self._cw.vreg['actions'].possible_actions(
            self._cw, rset=self.cw_rset, view=self.cw_extra_kwargs['view'])
        html = []
        for action in actions.get('useractions', ()):
            self.render_actions(html.append, action)
        if actions.get('useractions') and actions.get('siteactions'):
            html.append(self.divider_html)
        for action in actions.get('siteactions', ()):
            self.render_actions(html.append, action)
        ddb = AUSDropDownBox(self._cw.user.login, u'\n'.join(html), klass='icon-user')
        ddb.render(w=w)

    def render_actions(self, w, action):
        w(u'<li>')
        self.action_link(action).render(w=w)
        w(u'</li>')


class AUSDropDownBox(utils.DropDownBox):
    def render(self, w):
        if not len(self.actions):
            return u''
        w(self.ul_template % {'title': self.title,
                              'actions': self.actions,
                              'klass': self.klass})


class BSRQLInputForm(basecomponents.RQLInputForm):
    """build the rql input form, usually displayed in the header"""
    __regid__ = 'rqlinput'
    cw_property_defs = basecomponents.VISIBLE_PROP_DEF
    visible = False
    formdef = u"""<form id="rqlinput" action="%(action)s" class="%(css_class)s" role="search">
    <input type="text" id="rql" name="rql" value="%(rql)s" placeholder="%(title)s" class="form-control"
       title="%(title)s" tabindex="%(ind)s" accesskey="q" />
    %(hidden)s</form>"""

    def call(self, view=None):
        req = self._cw
        if hasattr(view, 'filter_box_context_info'):
            rset = view.filter_box_context_info()[0]
        else:
            rset = self.cw_rset
        # display multilines query as one line
        rql = rset is not None and rset.printable_rql() or req.form.get('rql', '')
        rql = rql.replace(u"\n", u" ")
        rql_suggestion_comp = self._cw.vreg['components'].select_or_none('rql.suggestions', self._cw)
        if rql_suggestion_comp is not None:
            # enable autocomplete feature only if the rql
            # suggestions builder is available
            self._cw.add_css('jquery.ui.css')
            self._cw.add_js(('cubicweb.ajax.js', 'jquery.ui.js'))
            self._cw.add_onload('$("#rql").autocomplete({source: "%s"});'
                                % (req.build_url('json', fname='rql_suggest')))
        hidden = u''
        if req.search_state[0] != 'normal':
            hidden = u'<input type="hidden" name="__mode" value="%s"/>' % \
                           ':'.join(req.search_state[1])
        self.w(self.formdef % {'css_class': not self.cw_propval('visible') and 'hidden' or '',
                               'action': req.build_url('view'),
                               'rql': xml_escape(rql),
                               'title': req._('full text or RQL query'),
                               'ind': req.next_tabindex() if not CW_325 else u'',
                               'hidden': hidden})


def registration_callback(vreg):
    components = (
        (BSAuthenticatedUserStatus, basecomponents.AuthenticatedUserStatus),
        (BSRQLInputForm, basecomponents.RQLInputForm),
        )
    vreg.register_all(globals().values(), __name__,
                      [new for (new, old) in components])
    for new, old in components:
        vreg.register_and_replace(new, old)
    vreg.unregister(basecomponents.AnonUserStatusLink)



"""fluid_design_system implementation of ibreadcrumbs

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb.entity import Entity
from cubicweb.web.views import ibreadcrumbs


ibreadcrumbs.BreadCrumbEntityVComponent.css_class = u'breadcrumb'


@monkeypatch(ibreadcrumbs.BreadCrumbEntityVComponent)
def open_breadcrumbs(self, w):
    w(u'<ul id="breadcrumbs" class="%s">' % self.css_class)


@monkeypatch(ibreadcrumbs.BreadCrumbEntityVComponent)
def close_breadcrumbs(self, w):
    w(u'</ul>')


@monkeypatch(ibreadcrumbs.BreadCrumbEntityVComponent)
def render_breadcrumbs(self, w, contextentity, path):
    root = path.pop(0)
    if isinstance(root, Entity):
        w(u'<li>')
        w(self.link_template % (self._cw.build_url(root.__regid__),
                                root.dc_type('plural')))
        w(u'</li>')
    liclass = u' class="active"' if not path else u''
    w(u'<li%s>' % liclass)
    self.wpath_part(w, root, contextentity, not path)
    w(u'</li>')
    for i, parent in enumerate(path):
        last = i == len(path) - 1
        liclass = u' class="active"' if last else u''
        w(u'<li%s>' % liclass)
        self.wpath_part(w, parent, contextentity, last)
        w(u'</li>')


@monkeypatch(ibreadcrumbs.BreadCrumbAnyRSetVComponent)
def render(self, w, **kwargs):
    self.open_breadcrumbs(w)
    w(u'<li class="active">')
    w(self._cw._('search'))
    w(u'</li>')
    self.close_breadcrumbs(w)



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of navigation components

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from logilab.common.decorators import monkeypatch

from logilab.mtconverter import xml_escape

from cubicweb.view import View
from cubicweb.web.component import NavigationComponent

from cubicweb.web.views.navigation import (NextPrevNavigationComponent,
                                           SortedNavigation,
                                           PageNavigation,
                                           PageNavigationSelect)
from cubicweb_fluid_design_system import CW_323

__docformat__ = "restructuredtext en"

NavigationComponent.page_link_templ = u'<li><a href="%s" title="%s">%s</a></li>'
NavigationComponent.selected_page_link_templ = u'<li class="active"><a href="%s" title="%s">%s</a></li>'
NavigationComponent.previous_page_link_templ = NavigationComponent.page_link_templ
NavigationComponent.next_page_link_templ = NavigationComponent.page_link_templ
NavigationComponent.no_previous_page_link = u'<li class="disabled"><a href="#">«</a></li>'
NavigationComponent.no_next_page_link = u'<li class="disabled"><a href="#">»</a></li>'
NavigationComponent.no_content_prev_link = u'«'
NavigationComponent.no_content_next_link = u'»'


@monkeypatch(NavigationComponent)
def render_link_back_to_pagination(self, w):
    "allow to come back to the paginated view"
    req = self._cw
    params = dict(req.form)
    basepath = req.relative_path(includeparams=False)
    del params['__force_display']
    url = self.page_url(basepath, params)
    w(u'<ul class="pagination"><li>'
      u'<span><a href="%s">%s</a></span>'
      u'</li></ul>'
      % (xml_escape(url),
         req._('back to pagination (%s results)') % self.page_size))


@monkeypatch(NavigationComponent)
def render_link_display_all(self, w):
    "make a link to see them all"
    req = self._cw
    params = dict(req.form)
    self.clean_params(params)
    basepath = req.relative_path(includeparams=False)
    params['__force_display'] = 1
    params['__fromnavigation'] = 1
    url = self.page_url(basepath, params)
    w(u'<ul class="pagination pull-right"><li>'
      u'<span><a href="%s">%s</a></span>'
      u'</li></ul>'
      % (xml_escape(url), req._('show %s results') % len(self.cw_rset)))


@monkeypatch(SortedNavigation)
def write_links(self, basepath, params, blocklist):
    """Return HTML for the whole navigation: `blocklist` is a list of HTML
    snippets for each page, `basepath` and `params` will be necessary to
    build previous/next links.
    """
    self.w(u'<ul class="pagination">')
    self.w(u'%s&#160;' % self.previous_link(basepath, params))
    self.w(u'%s' % u''.join(blocklist))
    self.w(u'&#160;%s' % self.next_link(basepath, params))
    self.w(u'</ul>')


@monkeypatch(PageNavigation)
def call(self):
    """displays a resultset by page"""
    params = dict(self._cw.form)
    self.clean_params(params)
    basepath = self._cw.relative_path(includeparams=False)
    self.w(u'<ul class="pagination">')
    self.w(u'%s&#160;' % self.previous_link(basepath, params))
    self.w(u'&#160;'.join(self.iter_page_links(basepath, params)))
    self.w(u'&#160;%s' % self.next_link(basepath, params))
    self.w(u'</ul>')


NextPrevNavigationComponent.prev_icon = u'&#8592;'
NextPrevNavigationComponent.next_icon = u'&#8594;'


@monkeypatch(NextPrevNavigationComponent)
# Should be better done, but this works
def render_body(self, w):
    w(u'<div class="prevnext row">')
    w(u'<ul class="pager">')
    self.prevnext(w)
    w(u'</ul>')
    w(u'</div>')
    # w(u'<div class="clear"></div>')


@monkeypatch(NextPrevNavigationComponent)
def prevnext_div(self, w, type, cssclass, url, title, content):
    csscls = {'prev': 'previous', 'next': 'next'}
    w(u'<li class="%s">' % csscls[type])
    w(u'<a href="%s" title="%s">%s</a>' % (xml_escape(url),
                                           xml_escape(title),
                                           content))
    w(u'</li>')
    self._cw.html_headers.add_raw(
        '<link rel="%s" href="%s" />' % (type, xml_escape(url)))


def do_paginate(view, rset=None, w=None, show_all_option=True, page_size=None):
    """write pages index in w stream (default to view.w) and then limit the
    result set (default to view.rset) to the currently displayed page if we're
    not explicitly told to display everything (by setting __force_display in
    req.form)
    """
    req = view._cw
    if rset is None:
        rset = view.cw_rset
    if w is None:
        w = view.w
    nav = req.vreg['components'].select_or_none(
        'navigation', req, rset=rset, page_size=page_size, view=view)
    if nav:
        if w is None:
            w = view.w
        if req.form.get('__force_display'):
            # allow to come back to the paginated view
            params = dict(req.form)
            basepath = req.relative_path(includeparams=False)
            del params['__force_display']
            url = nav.page_url(basepath, params)
            w(u'<span><a href="%s">%s</a></span>\n'
              % (xml_escape(url),
                 req._('back to pagination (%s results)') % nav.page_size))
        else:
            # get boundaries before component rendering
            start, stop = nav.page_boundaries()
            nav.render(w=w)
            params = dict(req.form)
            nav.clean_params(params)
            # make a link to see them all
            if show_all_option:
                basepath = req.relative_path(includeparams=False)
                params['__force_display'] = 1
                params['__fromnavigation'] = 1
                url = nav.page_url(basepath, params)
                w(u'<span><a href="%s">%s</a></span>\n'
                  % (xml_escape(url), req._('show %s results') % len(rset)))
            rset.limit(offset=start, limit=stop-start, inplace=True)  # noqa


def paginate(view, show_all_option=True, w=None, page_size=None, rset=None):
    """paginate results if the view is paginable
    """
    if view.paginable:
        do_paginate(view, rset, w, show_all_option, page_size)

# monkey patch base View class to add a .paginate([...])
# method to be called to write pages index in the view and then limit the result
# set to the current page


if not CW_323:
    View.do_paginate = do_paginate
    View.paginate = paginate
View.handle_pagination = False


@monkeypatch(PageNavigationSelect)  # noqa: F811
def call(self):
    params = dict(self._cw.form)
    self.clean_params(params)
    basepath = self._cw.relative_path(includeparams=False)
    w = self.w
    w(u'<ul class="pagination">')
    w(self.previous_link(basepath, params))
    start, stop = self.page_boundaries()
    w(u'<li><span><a class="dropdown-toggle" data-toggle="dropdown" href="#">'
        '%s<span class="caret"></span></a>' % self.index_display(start, stop - 1))
    w(u'<ul class="dropdown-menu" role="menu">')
    for option in self.iter_page_links(basepath, params):
        w(option)
    w(u'</ul></span></li>')
    w(u'&#160;%s' % self.next_link(basepath, params))
    w(u'</ul>')


PageNavigationSelect.page_link_templ = u'<li><a href="%s" title="%s">%s</a></li>'
PageNavigationSelect.selected_page_link_templ = u'<li class="disabled"><a href="%s" title="%s">%s</a></li>'



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of base templates

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb import _
from cubicweb.web import formwidgets as fw
from cubicweb.web.views import basetemplates, basecomponents, actions

from cubicweb_fluid_design_system import CW_325

from cubicweb.schema import display_name
from cubicweb.utils import UStringIO

from cubicweb.web.views.boxes import SearchBox


basetemplates.LogForm.needs_css = ()
basetemplates.LogForm.form_buttons = [
    fw.ResetButton(label=_('cancel'),
                   attrs={'class': 'btn btn-default',
                          'data-dismiss': 'modal'}),
    fw.SubmitButton(label=_('log in'),
                    attrs={'class': 'btn btn-primary'})]

basetemplates.LogForm.form_renderer_id = 'modal-form-renderer'


@monkeypatch(basetemplates.LogFormView)
def call(self, id, klass, title=True, showmessage=True, showonload=True):
    w = self.w
    stitle = '&#160;'
    if title:
        stitle = self._cw.property_value('ui.site-title')
        if stitle:
            stitle = xml_escape(stitle)
    config = self._cw.vreg.config
    if config['auth-mode'] != 'http':
        self.login_form(id, "loginModal", stitle,
                        showmessage, showonload)  # Cookie authentication
    if (CW_325 or self._cw.https) and config.anonymous_user()[0] and config.get('https-deny-anonymous'):
        path = xml_escape(config['base-url'] + self._cw.relative_path())
        w(u'<div class="loginMessage alert"><a href="%s">%s</a></div>\n'
          % (path, self._cw._('No account? Try public access at %s') % path))


@monkeypatch(basetemplates.LogFormView)
def login_form(self, id, modal_id, title, showmessage, showonload):
    cw = self._cw
    form = cw.vreg['forms'].select('logform', cw)
    if showonload:
        # remove cancel button on ?vid=login
        form.form_buttons = form.form_buttons[1:]
    if cw.vreg.config['allow-email-login']:
        label = cw._('login or email')
    else:
        label = cw.pgettext('CWUser', 'login')
    form.field_by_name('__login').label = label
    form.render(w=self.w, table_class='', display_progress_div=False,
                modal_id=modal_id,
                title=title,
                showmessage=showmessage,
                showonload=showonload)
    cw.html_headers.add_onload('jQuery("#__login:visible").focus()')


HTML5 = u'<!DOCTYPE html>'

basetemplates.TheMainTemplate.doctype = HTML5

# options which can be changed freely
basetemplates.TheMainTemplate.twbs_container_cls = 'container-fluid'
basetemplates.TheMainTemplate.twbs_col_cls = 'col-xs-'
basetemplates.TheMainTemplate.twbs_col_size = 2

# options which require recompiling bootstrap.css from source
basetemplates.TheMainTemplate.twbs_grid_columns = 12


@monkeypatch(basetemplates.TheMainTemplate)  # noqa: F811
def call(self, view):
    self.set_request_content_type()
    self.template_header(self.content_type, view)
    self.template_page_content(view)


@monkeypatch(basetemplates.TheMainTemplate)
def template_header(self, content_type, view=None, page_title='', additional_headers=()):
    page_title = page_title or view.page_title()
    additional_headers = additional_headers or view.html_headers()
    self.template_html_header(content_type, page_title, additional_headers)


@monkeypatch(basetemplates.TheMainTemplate)
def template_html_header(self, content_type, page_title,
                         additional_headers=()):
    w = self.whead
    self.write_doctype()
    self._cw.html_headers.define_var('BASE_URL', self._cw.base_url())
    self._cw.html_headers.define_var('DATA_URL', self._cw.datadir_url)
    w(u'<meta http-equiv="content-type" content="%s; charset=%s"/>\n'
      % (content_type, self._cw.encoding))
    w(u'<meta name="viewport" content="initial-scale=1.0, '
      u'maximum-scale=1.0, width=device-width"/>')
    w(u'\n'.join(additional_headers) + u'\n')
    self.wview('htmlheader', rset=self.cw_rset)
    if page_title:
        w(u'<title>%s</title>\n' % xml_escape(page_title))


@monkeypatch(basetemplates.TheMainTemplate)
def template_page_content(self, view):
    w = self.w
    self.w(u'<body>\n')
    self.wview('header', rset=self.cw_rset, view=view)
    w(u'<div id="page" class="%s">\n' % self.twbs_container_cls)
    w(u'<div class="row">\n')
    left_boxes = list(self._cw.vreg['ctxcomponents'].poss_visible_objects(
        self._cw, rset=self.cw_rset, view=view, context='left'))
    right_boxes = list(self._cw.vreg['ctxcomponents'].poss_visible_objects(
        self._cw, rset=self.cw_rset, view=view, context='right'))
    nb_boxes = int(bool(left_boxes)) + int(bool(right_boxes))
    content_cols = self.twbs_grid_columns
    if nb_boxes:
        content_cols = self.twbs_grid_columns - self.twbs_col_size * nb_boxes
    self.nav_column(view, left_boxes, 'left')
    self.content_column(view, content_cols)
    self.nav_column(view, right_boxes, 'right')
    self.w(u'</div>\n')  # closes class=row
    self.w(u'</div>\n')  # closes id="page" from template_page_content
    self.template_footer(view)
    self.w(u'</body>\n')


@monkeypatch(basetemplates.TheMainTemplate)  # noqa: F811
def get_components(self, view, context):
    ctxcomponents = self._cw.vreg['ctxcomponents']
    return ctxcomponents.poss_visible_objects(self._cw,
                                              rset=self.cw_rset,
                                              view=view,
                                              context=context)


@monkeypatch(basetemplates.TheMainTemplate)
def state_header(self):
    state = self._cw.search_state
    if state[0] == 'normal':
        return
    _ = self._cw._
    value = self._cw.view('oneline', self._cw.eid_rset(state[1][1]))
    target, eid, r_type, searched_type = self._cw.search_state[1]
    cancel_link = u'''<a href="%(url)s" role="button"
    class="btn btn-default" title="%(title)s">%(title)s</a>''' % {
        'url': self._cw.build_url(str(eid),
                                  vid='edition', __mode='normal'),
        'title': _('cancel')}
    msg = ' '.join((_("searching for"),
                    '<strong>"%s"</strong>' %
                    display_name(self._cw, state[1][3]),
                    _("to associate with"), value,
                    _("by relation"),
                    '<strong>"%s"</strong>' %
                    display_name(self._cw, state[1][2], state[1][0]),
                    cancel_link))
    return self.w(u'<div class="alert alert-info">%s</div>' % msg)


@monkeypatch(basetemplates.TheMainTemplate)
def nav_column(self, view, boxes, context):
    if boxes:
        stream = UStringIO()
        for box in boxes:
            box.render(w=stream.write, view=view)
        html = stream.getvalue()
        if html:
            # only display aside columns if html availble
            self.w(u'<aside id="aside-main-%s" class="%s%s cwjs-aside">\n' %
                   (context, self.twbs_col_cls, self.twbs_col_size))
            self.w(html)
            self.w(u'</aside>\n')
    return len(boxes)


@monkeypatch(basetemplates.TheMainTemplate)
def content_column(self, view, content_cols):
    w = self.w
    w(u'<div id="main-center" class="%(prefix)s%(col)s" role="main">' % {
        'prefix': self.twbs_col_cls, 'col': content_cols})
    components = self._cw.vreg['components']
    self.content_components(view, components)
    w(u'<div id="pageContent">')
    self.content_header(view)
    vtitle = self._cw.form.get('vtitle')
    if vtitle:
        w(u'<div class="vtitle">%s</div>\n' % xml_escape(vtitle))
    self.state_header()
    self.content_navrestriction_components(view, components)
    nav_html = UStringIO()
    if view and not view.handle_pagination:
        view.paginate(w=nav_html.write)
    w(nav_html.getvalue())
    w(u'<div id="contentmain">\n')
    view.render(w=w)
    w(u'</div>\n')  # closes id=contentmain
    w(nav_html.getvalue())
    self.content_footer(view)
    w(u'</div>\n')  # closes div#pageContent
    w(u'</div>\n')  # closes div.%(prefix)s-%(col)s


@monkeypatch(basetemplates.TheMainTemplate)
def content_components(self, view, components):
    """TODO : should use context"""
    rqlcomp = components.select_or_none('rqlinput', self._cw, rset=self.cw_rset)
    if rqlcomp:
        rqlcomp.render(w=self.w, view=view)
    msgcomp = components.select_or_none('applmessages', self._cw, rset=self.cw_rset)
    if msgcomp:
        msgcomp.render(w=self.w)


@monkeypatch(basetemplates.TheMainTemplate)
def content_navrestriction_components(self, view, components):
    # display entity type restriction component
    etypefilter = components.select_or_none(
        'etypenavigation', self._cw, rset=self.cw_rset)
    if etypefilter and etypefilter.cw_propval('visible'):
        etypefilter.render(w=self.w)


@monkeypatch(basetemplates.TheMainTemplate)
def template_footer(self, view=None):
    self.wview('footer', rset=self.cw_rset, view=view)


# main header

basecomponents.ApplLogo.context = 'header-logo'
# use basecomponents.ApplicationName.visible = False
basecomponents.ApplicationName.context = 'header-left'
basecomponents.ApplLogo.order = 1
basecomponents.ApplicationName.order = 10
basecomponents.CookieLoginComponent.order = 10
basecomponents.AuthenticatedUserStatus.order = 5
SearchBox.order = -1
SearchBox.context = 'header-right'
SearchBox.layout_id = 'simple-layout'


@monkeypatch(basetemplates.HTMLPageHeader)  # noqa: F811
def call(self, view, **kwargs):
    self.main_header(view)
    self.breadcrumbs(view)


def get_components(self, view, context):  # noqa: F811
    ctxcomponents = self._cw.vreg['ctxcomponents']
    return ctxcomponents.poss_visible_objects(self._cw,
                                              rset=self.cw_rset,
                                              view=view,
                                              context=context)


basetemplates.HTMLPageHeader.get_components = get_components
basetemplates.HTMLPageHeader.css = {
    'navbar-extra': 'navbar-default',
    'breadcrumbs': 'cw-breadcrumb',
    'container-cls': basetemplates.TheMainTemplate.twbs_container_cls,
    'header-left': '',
    'header-right': 'navbar-right',
}


@monkeypatch(basetemplates.HTMLPageHeader)
def main_header(self, view):
    w = self.w
    w(u'<nav class="navbar %s" role="banner">' % self.css['navbar-extra'])
    w(u'<div class="%s">' % self.css['container-cls'])
    self.display_navbar_header(w, view)
    w(u'<div id="tools-group" class="collapse navbar-collapse">')
    self.display_header_components(w, view, 'header-left')
    self.display_header_components(w, view, 'header-right')
    w(u'</div></div></nav>')


def display_navbar_header(self, w, view):
    w(u'''<div class="navbar-header">
    <button class="navbar-toggle" data-target="#tools-group" data-toggle="collapse" type="button">
    <span class="sr-only">%(toggle_label)s</span>
    <span class="icon-bar"></span>
    <span class="icon-bar"></span>
    <span class="icon-bar"></span>
    </button>''' % {'toggle_label': self._cw._('Toggle navigation')})
    components = self.get_components(view, context='header-logo')
    if components:
        for component in components:
            component.render(w=w)
    w(u'</div>')


basetemplates.HTMLPageHeader.display_navbar_header = display_navbar_header


def display_header_components(self, w, view, context):
    components = self.get_components(view, context=context)
    if components:
        w(u'<ul class="nav navbar-nav %s">' % self.css[context])
        for component in components:
            w(u'<li>')
            component.render(w=w)
            w(u'</li>')
        w(u'</ul>')


basetemplates.HTMLPageHeader.display_header_components = display_header_components


@monkeypatch(basetemplates.HTMLPageHeader)
def breadcrumbs(self, view):
    components = self.get_components(view, context='header-center')
    if components:
        self.w(u'<nav role="navigation" class="%s">' %
               self.css.get('breadcrumbs', 'breadcrumbs-defaul'))
        for component in components:
            component.render(w=self.w)
        self.w(u'</nav>')


@monkeypatch(basetemplates.HTMLContentFooter)  # noqa: F811
def call(self, view, **kwargs):
    components = self._cw.vreg['ctxcomponents'].poss_visible_objects(
        self._cw, rset=self.cw_rset, view=view, context='navbottom')
    if components:
        # the row is needed here to correctly put the HTML flux
        self.w(u'<div id="contentfooter">')
        for comp in components:
            comp.render(w=self.w, view=view)
        self.w(u'</div>')


@monkeypatch(basetemplates.HTMLPageFooter)  # noqa: F811
def call(self, **kwargs):
    self.w(u'<footer id="pagefooter" role="contentinfo">')
    self.footer_content()
    self.w(u'</footer>\n')


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__)
    vreg.unregister(actions.CancelSelectAction)



# copyright 2013-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <https://www.gnu.org/licenses/>.
"""fluid_design_system implementation of formrenderers"""

__docformat__ = "restructuredtext en"

from warnings import warn

from logilab.common.decorators import monkeypatch

from cubicweb import tags
from cubicweb.utils import support_args
from cubicweb.web.views import formrenderers


@monkeypatch(formrenderers.EntityCompositeFormRenderer)
def render_fields(self, w, form, values):
    if form.parent_form is None:
        # We should probably take those CSS classes to uiprops.py
        w(u'<table class="table table-striped table-bordered table-condensed">')
        # get fields from the first subform with something to display (we
        # may have subforms with nothing editable that will simply be
        # skipped later)
        for subform in form.forms:
            subfields = [field for field in subform.fields
                         if field.is_visible()]
            if subfields:
                break
        if subfields:
            # main form, display table headers HTML5
            w(u'<thead>')
            w(u'<tr>')
            w(u'<th>%s</th>' %
              tags.input(type='checkbox',
                         title=self._cw._('toggle check boxes'),
                         onclick="setCheckboxesState('eid', null, this.checked)"))
            for field in subfields:
                w(u'<th>%s</th>' % formrenderers.field_label(form, field))
            w(u'</tr>')
            w(u'</thead>')
    super(formrenderers.EntityCompositeFormRenderer, self).render_fields(w, form, values)
    if form.parent_form is None:
        w(u'</table>')
        if self._main_display_fields:
            super(formrenderers.EntityCompositeFormRenderer, self)._render_fields(
                self._main_display_fields, w, form)


formrenderers.FormRenderer.button_bar_class = u'form-group'


class ModalFormRenderer(formrenderers.FormRenderer):
    __regid__ = 'modal-form-renderer'
    button_bar_class = 'modal-footer'

    def open_form(self, form, values, **attrs):
        showmessage = values.get('showmessage')
        showonload = values.get('showonload', False)
        if showonload:
            # show the `modal` dialog : initialized in
            # basecomponents.CookieLoginComponent by showonload=True
            self._cw.add_onload("$('#%s').modal('show')" % values['modal_id'])
            data_backdrop, data_keyboard = u'static', u'false'
        else:
            data_backdrop, data_keyboard = u'true', u'true'

        html = [u'<div class="%(class)s" id="%(id)s" tabindex="-1" role="dialog" '
                u'data-backdrop="%(backdrop)s" data-keyboard="%(keyboard)s">\n'
                u'<div class="modal-dialog">'
                u'<div class="modal-content">' % {
                    'id': values['modal_id'],
                    'class': u'modal fade in' if showmessage else u'modal',
                    'backdrop': data_backdrop, 'keyboard': data_keyboard}]
        html.append(super(ModalFormRenderer, self).open_form(form, values, **attrs))
        html.append(u'<div class="modal-header">')
        if not showonload:
            # add cancel button when modal triggered by CookieLoginComponent
            html.append(u'<button type="button" class="close" data-dismiss="modal" '
                        'aria-hidden="true">&#215;</button>')
        html.append(
            u'<div class="modal-title">%s</div>\n</div>\n' % values.get('title', u''))
        return u'\n'.join(html)

    def render_content(self, w, form, values):
        self._cw.add_onload("$('.close').click(function () {$(this).parent().removeClass('in');});")
        w(u'<div class="modal-body">')
        if values.get('showmessage'):
            message = self._cw.message
            if message:
                w(u'<div class="alert alert-danger in">%s'
                  u'<button class="close" data-dismiss="alert">x</button></div>' % message)
        if self.display_progress_div:
            w(u'<div id="progress">%s</div>' % self._cw._('validating...'))
        w(u'\n<fieldset>\n')
        self.render_fields(w, form, values)
        w(u'\n</fieldset>\n')
        w(u'</div>')
        self.render_buttons(w, form)

    def close_form(self, form, values):
        html = [super(ModalFormRenderer, self).close_form(form, values)]
        html.append('</div></div></div>')  # close modal-content, modal-dialog, ...
        return u'\n'.join(html)


@monkeypatch(formrenderers.FormRenderer)
def render_help(self, form, field):
    """display help in the form
    """
    help = []
    descr = field.help
    if callable(descr):
        if support_args(descr, 'form', 'field'):
            descr = descr(form, field)
        else:
            warn("[3.10] field's help callback must now take form "
                 "and field as argument (%s)" % field, DeprecationWarning)
            descr = descr(form)
    if descr:
        help.append('<p class="help-block">%s</p>' % self._cw._(descr))
    example = field.example_format(self._cw)
    if example:
        help.append('<p class="form-control-static">(%s: %s)</p>'
                    % (self._cw._('sample format'), example))
    return u'&#160;'.join(help)


@monkeypatch(formrenderers.FormRenderer)
def error_message(self, form):
    """return formatted error message

    This method should be called once inlined field errors has been consumed
    """
    req = self._cw
    errex = form.form_valerror
    # get extra errors
    if errex is not None:
        errormsg = req._('please correct the following errors:')
        errors = form.remaining_errors()
        if errors:
            if len(errors) > 1:
                templstr = u'<li>%s</li>'
            else:
                templstr = u'&#160;%s'
            for field, err in errors:
                if field is None:
                    errormsg += templstr % err
                else:
                    errormsg += templstr % '%s: %s' % (req._(field), err)
            if len(errors) > 1:
                errormsg = '<ul>%s</ul>' % errormsg
        return u'<div class="alert alert-danger">%s</div>' % errormsg
    return u''


@monkeypatch(formrenderers.FormRenderer)
def _render_fields(self, fields, w, form):
    """render form fields
    """
    byfieldset = {}
    for field in fields:
        byfieldset.setdefault(field.fieldset, []).append(field)
    if form.fieldsets_in_order:
        fieldsets = form.fieldsets_in_order
    else:
        fieldsets = byfieldset
    for fieldset in list(fieldsets):
        try:
            fields = byfieldset.pop(fieldset)
        except KeyError:
            self.warning('no such fieldset: %s (%s)', fieldset, form)
            continue
        w(u'<fieldset>\n')
        if fieldset:
            w(u'<legend>%s</legend>' % self._cw.__(fieldset))
        for field in fields:
            error = form.field_error(field)
            control = not hasattr(field, 'control_field') or field.control_field
            w(u'<div id="%s-%s_row" class="form-group %s">' % (
                field.name, field.role, 'error' if error else ''))
            if self.display_label and field.label is not None:
                w(u'%s' % self.render_label(form, field))
            # Use full width for inlined forms, 9/12 for normal fields
            base_css = 'col-md-12' if hasattr(field, 'view') else 'col-md-9'
            if not control:
                # katia : 'control' class no longer exists in fluid_design_system 3.0.0
                # but is used here because of
                # 'form-horizontal' fluid_design_system 2.0.4 css
                # backport
                base_css += ' nomargin'
            w(u'<div class="%s">' % base_css)
            w(field.render(form, self))
            if error:
                self.render_error(w, error)
            if self.display_help:
                w(self.render_help(form, field))
            w(u'</div>')
            w(u'</div>')
        w(u'</fieldset>')
    if byfieldset:
        self.warning('unused fieldsets: %s', ', '.join(byfieldset))


@monkeypatch(formrenderers.FormRenderer)
def render_label(self, form, field):
    if field.label is None:
        return u''
    label = formrenderers.field_label(form, field)
    attrs = {'for': field.dom_id(form)}
    attrs['class'] = 'col-md-3 control-label'
    if field.required:
        attrs['class'] += ' required'
    return tags.label(label, **attrs)


@monkeypatch(formrenderers.FormRenderer)
def render_buttons(self, w, form):
    """render form's buttons
    """
    if not form.form_buttons:
        return
    w(u'<div class="%s">' % self.button_bar_class)
    for button in form.form_buttons:
        w(button.render(form))
    w(u'</div>')


@monkeypatch(formrenderers.EntityFormRenderer)
def open_form(self, form, values):
    attrs_fs_label = u''
    if self.main_form_title:
        attrs_fs_label = u'<h3>%s</h3>' % self._cw._(self.main_form_title)
    open_form = attrs_fs_label + super(formrenderers.EntityFormRenderer,
                                       self).open_form(form, values)
    return open_form


@monkeypatch(formrenderers.EntityFormRenderer)
def close_form(self, form, values):
    # needed to remove the '</div>' from the original method
    return super(formrenderers.EntityFormRenderer, self).close_form(form, values)


@monkeypatch(formrenderers.EntityFormRenderer)  # noqa: F811
def render_buttons(self, w, form):
    # needed to use our own monkeypatched FormRenderer.render_buttons() which
    # doesn't need to add a special case when there are 3 buttons to render
    super(formrenderers.EntityFormRenderer, self).render_buttons(w, form)


@monkeypatch(formrenderers.EntityInlinedFormRenderer)  # noqa: F811
def open_form(self, w, form, values):
    try:
        w(u'<div id="div-%(divid)s" onclick="%(divonclick)s">' % values)
    except KeyError:
        w(u'<div id="div-%(divid)s">' % values)
    else:
        w(u'<div id="notice-%s" class="notice text-info">'
          u'<span class="glyphicon glyphicon-info-sign"></span> %s</div>' % (
              values['divid'], self._cw._('click on the box to cancel the deletion')))
    w(u'<div class="iformBody">')


@monkeypatch(formrenderers.EntityInlinedFormRenderer)  # noqa: F811
def close_form(self, w, form, values):
    w(u'</div></div>')


@monkeypatch(formrenderers.EntityInlinedFormRenderer)  # noqa: F811
def render_fields(self, w, form, values):
    w(u'<fieldset id="fs-%(divid)s">' % values)
    fields = self._render_hidden_fields(w, form)
    w(u'</fieldset>')
    if fields:
        self._render_fields(fields, w, form)
    self.render_child_forms(w, form, values)



# pylint: disable=W0622
"""cubicweb-fluid-design-system application packaging information"""

modname = 'fluid_design_system'
distname = 'cubicweb-fluid-design-system'

numversion = (1, 7, 5)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
author = 'LOGILAB S.A. (Paris, FRANCE)'
author_email = 'contact@logilab.fr'
description = ''
web = 'https://www.cubicweb.org/project/%s' % distname

__depends__ = {
    'cubicweb': '>= 3.26.0',
    'six': '>= 1.12.O',
}
__recommends__ = {}

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Programming Language :: JavaScript',
]



"""fluid_design_system implementation of formwidgets

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb import tags, __version__ as cwver
from cubicweb.web import formwidgets

from cubicweb_fluid_design_system import CW_325


# Buttons
@monkeypatch(formwidgets.Button)
def render(self, form, field=None, renderer=None):
    label = form._cw._(self.label)
    attrs = self.attrs.copy()
    if not cwver.startswith('3.20'):
        # 3.21 commit b0417cacecd9
        attrs.setdefault('class', self.css_class)
    if self.cwaction:
        assert self.onclick is None
        attrs['onclick'] = "postForm('__action_%s', \'%s\', \'%s\')" % (
            self.cwaction, self.label, form.domid)
    elif self.onclick:
        attrs['onclick'] = self.onclick
    if self.name:
        attrs['name'] = self.name
        if self.setdomid:
            attrs['id'] = self.name
    if not CW_325:
        if self.settabindex and 'tabindex' not in attrs:
            attrs['tabindex'] = form._cw.next_tabindex()
    if self.icon:
        img = u'<i class="%s"> </i>' % self.icon
    else:
        img = u''
    return tags.button(img + xml_escape(label), escapecontent=False,
                       value=label, type=self.type, **attrs)



# copyright 2013-2021 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact https://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <https://www.gnu.org/licenses/>.
"""this is where you could register procedures for instance"""
__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb.web import formwidgets, stdmsgs

from cubicweb_fluid_design_system import monkeypatch_default_value

# put there monkeypatches of things that aren't reloaded automatically

# monkeypatches #############################################################

# keep the original `formwidgets.FieldWidget.attributes`
orig_attributes = formwidgets.FieldWidget.attributes

formwidgets.FieldWidget.css_class = 'form-control'

# Keep the `validateButton` as it is used in js (e.g `unfreezeFormButtons`)
formwidgets.ButtonInput.css_class = 'btn btn-default'
formwidgets.Button.css_class = 'btn btn-default validateButton'
formwidgets.SubmitButton.css_class = 'btn btn-primary validateButton'


@monkeypatch(formwidgets.FieldWidget)
def attributes(self, form, field):
    """ add the bootstrap `form-control` class to formwidgets.FieldWidget.attributes method
    """
    attrs = orig_attributes(self, form, field)
    if self.css_class:
        attrs['class'] = ' '.join((attrs.get('class', ''),
                                   self.css_class)).strip()
    return attrs

# in bootstrap 3.0.0 CheckBox, RadioBox and FileInput must not have
# form-control class


formwidgets.FileInput.attributes = orig_attributes
formwidgets.CheckBox.attributes = orig_attributes


# Buttons
stdmsgs.BUTTON_OK = (stdmsgs.BUTTON_OK[0], 'glyphicon glyphicon-ok')
stdmsgs.BUTTON_APPLY = (stdmsgs.BUTTON_APPLY[0], 'glyphicon glyphicon-cog')
stdmsgs.BUTTON_CANCEL = (stdmsgs.BUTTON_CANCEL[0], 'glyphicon glyphicon-remove')
stdmsgs.BUTTON_DELETE = (stdmsgs.BUTTON_DELETE[0], 'glyphicon glyphicon-trash')


# Override default form buttons to 'btn-class' and use bootstrap glyphicons.


monkeypatch_default_value(formwidgets.Button.__init__, 'label',
                          (stdmsgs.BUTTON_OK[0], 'glyphicon glyphicon-ok'))


# options #############################################################

options = (
    ('fluid_design_system_compatibility',
     {'type': 'yn',
      'default': False,
      'help': 'use compat css rules (enable if your application uses views '
      'that have not been ported to bootstrap)',
      'group': 'fluid_design_system', 'level': 2}),
)



"""
:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from logilab.common.decorators import monkeypatch

from cubicweb import view as cwview
from cubicweb.schema import display_name
from cubicweb.web.views.basetemplates import HTMLHeader
from cubicweb.web.views.boxes import ContextualBoxLayout, contextual

# do not wrap cell_calls with <div class="section">

cwview.View.add_div_section = False


@monkeypatch(cwview.View)
def field(self, label, value, row=True, show_label=True, w=None, tr=True,
          table=False):
    """read-only field"""
    if w is None:
        w = self.w
    w(u'<tr>')
    if show_label and label:
        if tr:
            label = display_name(self._cw, label)
        w(u'<th>%s</th>' % label)
    if not (show_label and label):
        w(u'<td colspan="2">%s</td>' % value)
    else:
        w(u'<td>%s</td>' % value)
    w(u'</tr>')


class BSHTMLHeader(HTMLHeader):

    def stylesheets(self):
        super(BSHTMLHeader, self).stylesheets()
        # add cw compatibility stylesheets
        if self._cw.vreg.config['fluid_design_system_compatibility']:
            for css in self._cw.uiprops['CW_COMPAT_STYLESHEETS']:
                self._cw.add_css(css, localfile=False)


class SimpleBoxContextFreeBoxLayout(ContextualBoxLayout):
    __select__ = ~contextual()
    cssclass = 'contextFreeBox'
    __regid__ = 'simple-layout'

    def render(self, w):
        if self.init_rendering():
            view = self.cw_extra_kwargs['view']
            view.render_body(w)


def registration_callback(vreg):
    components = (
        (BSHTMLHeader, HTMLHeader),
    )
    vreg.register_all(globals().values(), __name__,
                      [new for (new, old) in components])
    for new, old in components:
        vreg.register_and_replace(new, old)



"""fluid_design_system implementation of tag cube

:organization: Logilab
:copyright: 2015 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

try:
    from cubicweb_tag.views import SimilarityBox
except ImportError:
    pass
else:
    from logilab.common.decorators import monkeypatch
    from cubicweb import tags

    @monkeypatch(SimilarityBox)
    def render_body(self, w):
        # bs customization begins
        self._cw.view('list', self.cw_rset, w=w, klass='list-unstyled')
        # bs customization ends
        rql = self.rql % '' % {'x': self.entity.eid}
        title = self._cw._('entities similar to %s') % self.entity.dc_title()
        url = self._cw.build_url('view', rql=rql,
                                 vtitle=title)
        w(u'<div>[%s]</div>' % tags.a(self._cw._('see all'), href=url))



# -*- coding: utf-8 -*-
"""fluid_design_system implementation of startup views

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from logilab.common.textutils import unormalize
from logilab.mtconverter import xml_escape

from cubicweb.web.views import startup, uicfg

startup.ManageView.box_html = u'''<div class="panel panel-default">
    <div class="panel-heading">
        <div class="panel-title">%(title)s</div>
    </div>
    <div class="panel-body">%(content)s</div>
    </div>'''


@monkeypatch(startup.ManageView)
def manage_actions(self):
    allactions = self._cw.vreg['actions'].possible_actions(self._cw)
    if allactions.get('manage'):
        html = []
        w = html.append
        w(u'<ul>')
        for action in allactions['manage']:
            w(u'<li><a href="%s">%s</a></li>' % (
                action.url(), self._cw._(action.title)))
        w(u'</ul>')
        self.w(self.box_html % {'title': self._cw._('Manage'),
                                'content': u'\n'.join(html)})


@monkeypatch(startup.ManageView)
def startup_views(self):
    views = [v for v in self._cw.vreg['views'].possible_views(self._cw, None)
             if v.category == 'startupview'
             and v.__regid__ not in self.skip_startup_views]
    if not views:
        return
    html = []
    w = html.append
    w(u'<ul>')
    for v in sorted(views, key=lambda x: self._cw._(x.title)):
        w('<li><a href="%s">%s</a></li>' % (
            xml_escape(v.url()), xml_escape(self._cw._(v.title).capitalize())))
    w(u'</ul>')
    self.w(self.box_html % {'title': self._cw._('Startup views'),
                            'content': u'\n'.join(html)})


@monkeypatch(startup.ManageView)
def entities(self):
    schema = self._cw.vreg.schema
    eschemas = [eschema for eschema in schema.entities()
                if uicfg.indexview_etype_section.get(eschema) == 'application']
    if eschemas:
        html = self.entity_types_table(eschemas)
        self.w(self.box_html % {'title': self._cw._('Browse by entity type'),
                                'content': u'\n'.join(html)})


@monkeypatch(startup.ManageView)
def entity_types_table(self, eschemas):
    infos = sorted(self.entity_types(eschemas),
                   key=lambda t: unormalize(t[0]))
    q, r = divmod(len(infos), 2)
    html = []
    w = html.append
    w(u'<div class="row">')
    for links in (infos[:q + r], infos[q + r:]):
        if links:
            w(u'<div class="col-md-6">')
            w(u'<ul class="list-unstyled">')
            for (_, etypelink, addlink) in links:
                w('<li>%s %s</li>' % (addlink, etypelink))
            w(u'</ul>')
            w(u'</div>')
    w(u'</div>')
    return html



"""fluid_design_system implementation of cwproperty view

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from cubicweb.web.views.cwproperties import (
    SystemCWPropertiesForm, make_togglable_link
)


@monkeypatch(SystemCWPropertiesForm)
def wrap_main_form(self, group, label, form):
    label += u' <span class="caret"></span>'
    status = self._group_status(group)
    cssclass = 'panel-body %s' % status if status else 'panel-body'
    self.w(u'<div class="panel panel-default">'
           u'<div class="panel-heading">%s</div>\n' %
           (make_togglable_link('fieldset_' + group, label)))
    self.w(u'<div class="%s" id="fieldset_%s">' % (cssclass, group))
    self.w(form)
    self.w(u'</div>')
    self.w(u'</div>')


@monkeypatch(SystemCWPropertiesForm)
def wrap_grouped_form(self, group, label, objects):
    label += u' <span class="caret"></span>'
    status = self._group_status(group)
    cssclass = 'panel-body %s' % status if status else 'panel-body'
    self.w(u'<div class="panel panel-default">'
           u'<div class="panel-heading">%s</div>\n' %
           (make_togglable_link('fieldset_' + group, label)))
    self.w(u'<div class="%s" id="fieldset_%s">' % (cssclass, group))

    sorted_objects = sorted((self._cw.__('%s_%s' % (group, o)), o, f)
                            for o, f in objects.items())
    for label, oid, form in sorted_objects:
        self.wrap_object_form(group, oid, label, form)
    self.w(u'</div>')
    self.w(u'</div>')



# -*- coding: utf-8 -*-
"""fluid_design_system html helpers

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"


class DropDownBox(object):
    ul_template = u'''
    <a class="dropdown-toggle %(klass)s" data-toggle="dropdown" href="#">
    %(title)s<span class="caret"></span></a>
    <ul class="dropdown-menu" role="menu">%(actions)s</ul>'''

    li_template = u'<li>%(link)s<li>'

    def __init__(self, title, actions, klass=''):
        self.title = title
        self.actions = actions
        self.klass = klass

    def render(self, w):
        if not len(self.actions):
            return u''
        w(self.ul_template % {'title': self.title, 'klass': self.klass,
                              'actions': u''.join(self.render_items())})

    def render_items(self):
        for item in self.actions:
            yield self.li_template % {'link': self._item_value(item)}

    def _item_value(self, item):
        return item



"""fluid_design_system implementation of base debug views

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from time import strftime, localtime

from six import text_type

from logilab.common.decorators import monkeypatch

from logilab.mtconverter import xml_escape

from cubicweb import BadConnectionId
from cubicweb.web.views import debug


def dict_to_html(w, dict):
    # XHTML doesn't allow emtpy <ul> nodes
    if dict:
        w(u'<dl>')
        for key in sorted(dict):
            w(u'<dt>%s</dt><dd>%s</dd>' % (
                xml_escape(str(key)), xml_escape(repr(dict[key]))))
        w(u'</dl>')


@monkeypatch(debug.ProcessInformationView)
def call(self, **kwargs):
    req = self._cw
    dtformat = req.property_value('ui.datetime-format')
    _ = req._
    w = self.w
    repo = req.cnx.repo
    # generic instance information
    w(u'<h2>%s</h2>' % _('Instance'))
    w(u'<table class="table table-striped table-condensed">')
    for key, value in ((_('config type'), self._cw.vreg.config.name),
                       (_('config mode'), self._cw.vreg.config.mode),
                       (_('instance home'), self._cw.vreg.config.apphome)):
        w(u'<tr><th>%s</th><td>%s</td></tr>' % (key, value))
    w(u'</table>')
    vcconf = repo.get_versions()
    w(u'<h3>%s</h3>' % _('versions configuration'))
    w(u'<table class="table table-striped table-condensed">')
    w(u'<tr><th>%s</th><td>%s</td></tr>' % (
        'CubicWeb', vcconf.get('cubicweb', _('no version information'))))
    for cube in sorted(self._cw.vreg.config.cubes()):
        cubeversion = vcconf.get(cube, _('no version information'))
        w(u'<tr><th>%s</th><td>%s</td></tr>' % (
            cube, cubeversion))
    w(u'</table>')
    # repository information
    w(u'<h2>%s</h2>' % _('Repository'))
    w(u'<h3>%s</h3>' % _('resources usage'))
    w(u'<table class="table table-striped table-condensed">')
    stats = self._cw.call_service('repo_stats')
    for element in sorted(stats):
        w(u'<tr><th>%s</th><td>%s %s</td></tr>'
          % (element, xml_escape(text_type(stats[element])),
             element.endswith('percent') and '%' or ''))
    w(u'</table>')
    # web server information
    w(u'<h2>%s</h2>' % _('Web server'))
    w(u'<table class="table table-striped table-condensed">')
    w(u'<tr><th>%s</th><td>%s</td></tr>' % (
        _('base url'), req.base_url()))
    w(u'<tr><th>%s</th><td>%s</td></tr>' % (
        _('data directory url'), req.datadir_url))
    w(u'</table>')
    from cubicweb.web.application import SESSION_MANAGER
    if SESSION_MANAGER is not None and req.user.is_in_group('managers'):
        sessions = SESSION_MANAGER.current_sessions()
        w(u'<h3>%s</h3>' % _('opened web sessions'))
        if sessions:
            w(u'<ul>')
            for session in sessions:
                if hasattr(session, 'cnx'):
                    # cubicweb < 3.19
                    if not session.cnx:
                        w(u'<li>%s (NO CNX)</li>' % session.sessionid)
                        continue
                    try:
                        last_usage_time = session.cnx.check()
                    except BadConnectionId:
                        w(u'<li>%s (INVALID)</li>' % session.sessionid)
                        continue
                else:
                    # cubicweb >= 3.19
                    last_usage_time = session.mtime
                w(u'<li>%s (%s: %s)<br/>' % (
                    session.sessionid,
                    _('last usage'),
                    strftime(dtformat, localtime(last_usage_time))))
                dict_to_html(w, session.data)
                w(u'</li>')
            w(u'</ul>')
        else:
            w(u'<p>%s</p>' % _('no web sessions found'))


@monkeypatch(debug.RegistryView)  # noqa: F811
def call(self, **kwargs):
    self.w(u'<h2>%s</h2>' % self._cw._("Registry's content"))
    keys = sorted(self._cw.vreg)
    url = xml_escape(self._cw.url())
    self.w(u'<p>%s</p>\n' % ' - '.join('<a href="%s#%s">%s</a>'
                                       % (url, key, key) for key in keys))
    for key in keys:
        if key in ('boxes', 'contentnavigation'):  # those are bw compat registries
            continue
        self.w(u'<h3 id="%s">%s</h3>' % (key, key))
        if self._cw.vreg[key]:
            values = sorted(self._cw.vreg[key].items())
            self.wview('pyvaltable', pyvalue=[(key, xml_escape(repr(val)))
                                              for key, val in values])
        else:
            self.w(u'<p>Empty</p>\n')



"""fluid_design_system implementation of htmlwidgets

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch

from cubicweb.web import htmlwidgets
from cubicweb.web.component import Separator


@monkeypatch(Separator)
def render(self, w):
    w(u'<li class="divider"></li>')


def bwcompatible_render_item(w, item):
    if hasattr(item, 'render'):
        if getattr(item, 'newstyle', False):
            if isinstance(item, Separator):
                item.render(w)
            else:
                w(u'<li>')
                item.render(w)
                w(u'</li>')
        else:
            item.render(w)  # XXX displays <li> by itself
    else:
        w(u'<li>%s</li>' % item)


@monkeypatch(htmlwidgets.BoxMenu)
def _render(self):
    tag = u'li' if self.isitem else u'div'
    self.w(u'<%s class="dropdown">' % tag)
    self.w(u'<a class="dropdown-toggle" data-toggle="dropdown" href="#">'
           u'%s&nbsp;'
           u'<span class="caret"></span>'
           u'</a>' % self.label)
    self.w(u'<ul class="dropdown-menu">')
    for item in self.items:
        bwcompatible_render_item(self.w, item)
    self.w(u'</ul>')
    self.w(u'</%s>' % tag)



"""fluid_design_system implementation of baseviews

:organization: Logilab
:copyright: 2013 LOGILAB S.A. (Paris, FRANCE), license is LGPL.
:contact: https://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

__docformat__ = "restructuredtext en"

from logilab.common.decorators import monkeypatch
from six.moves import range

from cubicweb.web.views import baseviews

baseviews.ListView.listtag = 'ul'


@monkeypatch(baseviews.ListView)
def call(self, klass='list-striped', title=None, subvid=None, listid=None, **kwargs):
    """display a list of entities by calling their <item_vid> view

    :param listid: the DOM id to use for the root element
    """
    if subvid is None and 'subvid' in self._cw.form:
        subvid = self._cw.form.pop('subvid')  # consume it
    if listid:
        listid = u' id="%s"' % listid
    else:
        listid = u''
    if title:
        self.w(u'<div%s class="%s"><h4>%s</h4>\n' % (listid, klass or u'', title))
        self.w(u'<%s class="%s">\n' % (self.listtag, klass or u''))
    else:
        self.w(u'<%s%s class="%s">\n' % (self.listtag, listid, klass or u''))
    for i in range(self.cw_rset.rowcount):
        self.cell_call(row=i, col=0, vid=subvid, klass=klass, **kwargs)
    self.w(u'</%s>\n' % self.listtag)
    if title:
        self.w(u'</div>\n')


@monkeypatch(baseviews.ListView)
def cell_call(self, row, col=0, vid=None, klass=None, **kwargs):
    self.w(u'<li class="%s">' % (u'odd' if row % 2 else u'even'))
    self.wview(self.item_vid, self.cw_rset, row=row, col=col, vid=vid, **kwargs)
    self.w(u'</li>\n')


@monkeypatch(baseviews.SameETypeListView)  # noqa: F811
def call(self, klass='list-striped', **kwargs):
    w = self.w
    showtitle = kwargs.pop('showtitle', 'vtitle' not in self._cw.form)
    if showtitle:
        w(u'<h1>%s</h1>' % self.title)
    klass = u'' if klass is None else u' class="%s"' % klass
    w(u'<ul%s>\n' % klass)
    for i in range(len(self.cw_rset)):
        w(u'<li class="%s">' % (u'odd' if i % 2 else u'even'))
        self._cw.view(self.item_vid, self.cw_rset, row=i, col=0, w=w)
        w(u'</li>')
    w(u'</ul>\n')


@monkeypatch(baseviews.MetaDataView)  # noqa: F811
def cell_call(self, row, col):
    _ = self._cw._
    entity = self.cw_rset.get_entity(row, col)
    self.w(u'<p class="text-right"><small>')
    if self.show_eid:
        self.w(u'%s #%s - ' % (entity.dc_type(), entity.eid))
    if entity.modification_date != entity.creation_date:
        self.w(u'<span>%s</span> ' % _('latest update on'))
        self.w(u'<span class="value">%s</span>, '
               % self._cw.format_date(entity.modification_date))
    # entities from external source may not have a creation date (eg ldap)
    if entity.creation_date:
        self.w(u'<span>%s</span> ' % _('created on'))
        self.w(u'<span class="value">%s</span>'
               % self._cw.format_date(entity.creation_date))
    if entity.creator:
        if entity.creation_date:
            self.w(u' <span>%s</span> ' % _('by'))
        else:
            self.w(u' <span>%s</span> ' % _('created_by'))
        self.w(u'<span class="value">%s</span>' % entity.creator.name())
    if entity.cw_source:
        source = entity.cw_source[0]
        if source.name != 'system':
            self.w(u' (<span>%s</span>' % _('cw_source'))
            self.w(u' <span class="value">%s</span>)' % source.name)
    self.w(u'</small></p>')
